package org.eclipse.jdt.internal.compiler.lookup;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import org.eclipse.jdt.core.compiler.CharOperation;
import org.eclipse.jdt.internal.compiler.lookup.BinaryTypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.Binding;
import org.eclipse.jdt.internal.compiler.lookup.LookupEnvironment;
import org.eclipse.jdt.internal.compiler.lookup.ProblemReferenceBinding;
import org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.TypeConstants;
import org.eclipse.jdt.internal.compiler.lookup.UnresolvedReferenceBinding;
import org.eclipse.jdt.internal.compiler.util.HashtableOfPackage;
import org.eclipse.jdt.internal.compiler.util.HashtableOfType;

public class PackageBinding extends Binding implements TypeConstants {
  public long tagBits = 0L;
  
  public char[][] compoundName;
  
  org.eclipse.jdt.internal.compiler.lookup.PackageBinding parent;
  
  public LookupEnvironment environment;
  
  HashtableOfType knownTypes;
  
  HashtableOfPackage knownPackages;
  
  protected int defaultNullness = 0;
  
  protected PackageBinding() {}
  
  public PackageBinding(char[] topLevelPackageName, LookupEnvironment environment) {
    this(new char[][] { topLevelPackageName }, null, environment);
  }
  
  public PackageBinding(char[][] compoundName, org.eclipse.jdt.internal.compiler.lookup.PackageBinding parent, LookupEnvironment environment) {
    this.compoundName = compoundName;
    this.parent = parent;
    this.environment = environment;
    this.knownTypes = null;
    this.knownPackages = new HashtableOfPackage(3);
    if (compoundName != CharOperation.NO_CHAR_CHAR)
      checkIfNullAnnotationPackage(); 
  }
  
  public PackageBinding(LookupEnvironment environment) {
    this(CharOperation.NO_CHAR_CHAR, null, environment);
  }
  
  private void addNotFoundPackage(char[] simpleName) {
    this.knownPackages.put(simpleName, (org.eclipse.jdt.internal.compiler.lookup.PackageBinding)LookupEnvironment.TheNotFoundPackage);
  }
  
  private void addNotFoundType(char[] simpleName) {
    if (this.knownTypes == null)
      this.knownTypes = new HashtableOfType(25); 
    this.knownTypes.put(simpleName, (ReferenceBinding)LookupEnvironment.TheNotFoundType);
  }
  
  void addPackage(org.eclipse.jdt.internal.compiler.lookup.PackageBinding element) {
    if ((element.tagBits & 0x80L) == 0L)
      clearMissingTagBit(); 
    this.knownPackages.put(element.compoundName[element.compoundName.length - 1], element);
  }
  
  void addType(ReferenceBinding element) {
    if ((element.tagBits & 0x80L) == 0L)
      clearMissingTagBit(); 
    if (this.knownTypes == null)
      this.knownTypes = new HashtableOfType(25); 
    char[] name = element.compoundName[element.compoundName.length - 1];
    ReferenceBinding priorType = this.knownTypes.getput(name, element);
    if (priorType != null && priorType.isUnresolvedType() && !element.isUnresolvedType())
      ((UnresolvedReferenceBinding)priorType).setResolvedType(element, this.environment); 
    if (this.environment.globalOptions.isAnnotationBasedNullAnalysisEnabled && (
      element.isAnnotationType() || element instanceof UnresolvedReferenceBinding))
      checkIfNullAnnotationType(element); 
  }
  
  void clearMissingTagBit() {
    org.eclipse.jdt.internal.compiler.lookup.PackageBinding current = this;
    do {
      current.tagBits &= 0xFFFFFFFFFFFFFF7FL;
    } while ((current = current.parent) != null);
  }
  
  public char[] computeUniqueKey(boolean isLeaf) {
    return CharOperation.concatWith(this.compoundName, '/');
  }
  
  private org.eclipse.jdt.internal.compiler.lookup.PackageBinding findPackage(char[] name) {
    if (!this.environment.isPackage(this.compoundName, name))
      return null; 
    char[][] subPkgCompoundName = CharOperation.arrayConcat(this.compoundName, name);
    org.eclipse.jdt.internal.compiler.lookup.PackageBinding subPackageBinding = new org.eclipse.jdt.internal.compiler.lookup.PackageBinding(subPkgCompoundName, this, this.environment);
    addPackage(subPackageBinding);
    return subPackageBinding;
  }
  
  org.eclipse.jdt.internal.compiler.lookup.PackageBinding getPackage(char[] name) {
    org.eclipse.jdt.internal.compiler.lookup.PackageBinding binding = getPackage0(name);
    if (binding != null) {
      if (binding == LookupEnvironment.TheNotFoundPackage)
        return null; 
      return binding;
    } 
    if ((binding = findPackage(name)) != null)
      return binding; 
    addNotFoundPackage(name);
    return null;
  }
  
  org.eclipse.jdt.internal.compiler.lookup.PackageBinding getPackage0(char[] name) {
    return this.knownPackages.get(name);
  }
  
  ReferenceBinding getType(char[] name) {
    ReferenceBinding referenceBinding = getType0(name);
    if (referenceBinding == null && (
      referenceBinding = this.environment.askForType(this, name)) == null) {
      addNotFoundType(name);
      return null;
    } 
    if (referenceBinding == LookupEnvironment.TheNotFoundType)
      return null; 
    referenceBinding = (ReferenceBinding)BinaryTypeBinding.resolveType((TypeBinding)referenceBinding, this.environment, false);
    if (referenceBinding.isNestedType())
      return (ReferenceBinding)new ProblemReferenceBinding(new char[][] { name }, referenceBinding, 4); 
    return referenceBinding;
  }
  
  ReferenceBinding getType0(char[] name) {
    if (this.knownTypes == null)
      return null; 
    return this.knownTypes.get(name);
  }
  
  public Binding getTypeOrPackage(char[] name) {
    ReferenceBinding referenceBinding = getType0(name);
    if (referenceBinding != null && referenceBinding != LookupEnvironment.TheNotFoundType) {
      referenceBinding = (ReferenceBinding)BinaryTypeBinding.resolveType((TypeBinding)referenceBinding, this.environment, false);
      if (referenceBinding.isNestedType())
        return (Binding)new ProblemReferenceBinding(new char[][] { name }, referenceBinding, 4); 
      if ((referenceBinding.tagBits & 0x80L) == 0L)
        return (Binding)referenceBinding; 
    } 
    org.eclipse.jdt.internal.compiler.lookup.PackageBinding packageBinding = getPackage0(name);
    if (packageBinding != null && packageBinding != LookupEnvironment.TheNotFoundPackage)
      return packageBinding; 
    if (referenceBinding == null) {
      if ((referenceBinding = this.environment.askForType(this, name)) != null) {
        if (referenceBinding.isNestedType())
          return (Binding)new ProblemReferenceBinding(new char[][] { name }, referenceBinding, 4); 
        return (Binding)referenceBinding;
      } 
      addNotFoundType(name);
    } 
    if (packageBinding == null) {
      if ((packageBinding = findPackage(name)) != null)
        return packageBinding; 
      if (referenceBinding != null && referenceBinding != LookupEnvironment.TheNotFoundType)
        return (Binding)referenceBinding; 
      addNotFoundPackage(name);
    } 
    return null;
  }
  
  public final boolean isViewedAsDeprecated() {
    if ((this.tagBits & 0x400000000L) == 0L) {
      this.tagBits |= 0x400000000L;
      if (this.compoundName != CharOperation.NO_CHAR_CHAR) {
        ReferenceBinding packageInfo = getType(TypeConstants.PACKAGE_INFO_NAME);
        if (packageInfo != null) {
          packageInfo.initializeDeprecatedAnnotationTagBits();
          this.tagBits |= packageInfo.tagBits & 0x17FFFFF800000000L;
        } 
      } 
    } 
    return ((this.tagBits & 0x400000000000L) != 0L);
  }
  
  public final int kind() {
    return 16;
  }
  
  public int problemId() {
    if ((this.tagBits & 0x80L) != 0L)
      return 1; 
    return 0;
  }
  
  void checkIfNullAnnotationPackage() {
    LookupEnvironment env = this.environment;
    if (env.globalOptions.isAnnotationBasedNullAnalysisEnabled) {
      if (isPackageOfQualifiedTypeName(this.compoundName, env.getNullableAnnotationName()))
        env.nullableAnnotationPackage = this; 
      if (isPackageOfQualifiedTypeName(this.compoundName, env.getNonNullAnnotationName()))
        env.nonnullAnnotationPackage = this; 
      if (isPackageOfQualifiedTypeName(this.compoundName, env.getNonNullByDefaultAnnotationName()))
        env.nonnullByDefaultAnnotationPackage = this; 
    } 
  }
  
  private boolean isPackageOfQualifiedTypeName(char[][] packageName, char[][] typeName) {
    int length;
    if (typeName == null || (length = packageName.length) != typeName.length - 1)
      return false; 
    for (int i = 0; i < length; i++) {
      if (!CharOperation.equals(packageName[i], typeName[i]))
        return false; 
    } 
    return true;
  }
  
  void checkIfNullAnnotationType(ReferenceBinding type) {
    if (this.environment.nullableAnnotationPackage == this && 
      CharOperation.equals(type.compoundName, this.environment.getNullableAnnotationName())) {
      type.typeBits |= 0x40;
      if (!(type instanceof UnresolvedReferenceBinding))
        this.environment.nullableAnnotationPackage = null; 
    } else if (this.environment.nonnullAnnotationPackage == this && 
      CharOperation.equals(type.compoundName, this.environment.getNonNullAnnotationName())) {
      type.typeBits |= 0x20;
      if (!(type instanceof UnresolvedReferenceBinding))
        this.environment.nonnullAnnotationPackage = null; 
    } else if (this.environment.nonnullByDefaultAnnotationPackage == this && 
      CharOperation.equals(type.compoundName, this.environment.getNonNullByDefaultAnnotationName())) {
      type.typeBits |= 0x80;
      if (!(type instanceof UnresolvedReferenceBinding))
        this.environment.nonnullByDefaultAnnotationPackage = null; 
    } else {
      type.typeBits |= this.environment.getNullAnnotationBit(type.compoundName);
    } 
  }
  
  public char[] readableName() {
    return CharOperation.concatWith(this.compoundName, '.');
  }
  
  public String toString() {
    String str;
    if (this.compoundName == CharOperation.NO_CHAR_CHAR) {
      str = "The Default Package";
    } else {
      str = "package " + ((this.compoundName != null) ? CharOperation.toString(this.compoundName) : "UNNAMED");
    } 
    if ((this.tagBits & 0x80L) != 0L)
      str = String.valueOf(str) + "[MISSING]"; 
    return str;
  }
}
